@@ -22,12 +22,12 @@ use syn::{
     TypeReference, Variant as RustVariant, Visibility,
 };
 
-pub mod kw {
+pub(crate) mod kw {
     syn::custom_keyword!(Pin);
     syn::custom_keyword!(Result);
 }
 
-pub fn parse_items(
+pub(crate) fn parse_items(
     cx: &mut Errors,
     items: Vec<Item>,
     trusted: bool,
@@ -42,7 +42,7 @@ pub fn parse_items(
             },
             Item::Enum(item) => apis.push(parse_enum(cx, item, namespace)),
             Item::ForeignMod(foreign_mod) => {
-                parse_foreign_mod(cx, foreign_mod, &mut apis, trusted, namespace)
+                parse_foreign_mod(cx, foreign_mod, &mut apis, trusted, namespace);
             }
             Item::Impl(item) => match parse_impl(cx, item) {
                 Ok(imp) => apis.push(imp),
@@ -436,14 +436,11 @@ fn parse_foreign_mod(
 }
 
 fn parse_lang(abi: &Abi) -> Result<Lang> {
-    let name = match &abi.name {
-        Some(name) => name,
-        None => {
-            return Err(Error::new_spanned(
-                abi,
-                "ABI name is required, extern \"C++\" or extern \"Rust\"",
-            ));
-        }
+    let Some(name) = &abi.name else {
+        return Err(Error::new_spanned(
+            abi,
+            "ABI name is required, extern \"C++\" or extern \"Rust\"",
+        ));
     };
 
     match name.value().as_str() {
@@ -489,11 +486,7 @@ fn parse_extern_type(
     let type_token = foreign_type.type_token;
     let visibility = visibility_pub(&foreign_type.vis, type_token.span);
     let name = pair(namespace, &foreign_type.ident, cxx_name, rust_name);
-    let generics = Lifetimes {
-        lt_token: None,
-        lifetimes: Punctuated::new(),
-        gt_token: None,
-    };
+    let generics = extern_type_lifetimes(cx, foreign_type.generics);
     let colon_token = None;
     let bounds = Vec::new();
     let semi_token = foreign_type.semi_token;
@@ -611,7 +604,27 @@ fn parse_extern_fn(
                     });
                     continue;
                 }
-                return Err(Error::new_spanned(arg, "unsupported signature"));
+                if let Some(colon_token) = arg.colon_token {
+                    let ty = parse_type(&arg.ty)?;
+                    if let Type::Ref(reference) = ty {
+                        if let Type::Ident(ident) = reference.inner {
+                            receiver = Some(Receiver {
+                                pinned: reference.pinned,
+                                ampersand: reference.ampersand,
+                                lifetime: reference.lifetime,
+                                mutable: reference.mutable,
+                                var: Token![self](ident.rust.span()),
+                                colon_token,
+                                ty: ident,
+                                shorthand: false,
+                                pin_tokens: reference.pin_tokens,
+                                mutability: reference.mutability,
+                            });
+                            continue;
+                        }
+                    }
+                }
+                return Err(Error::new_spanned(arg, "unsupported method receiver"));
             }
             FnArg::Typed(arg) => {
                 let ident = match arg.pat.as_ref() {
@@ -622,45 +635,24 @@ fn parse_extern_fn(
                     _ => return Err(Error::new_spanned(arg, "unsupported signature")),
                 };
                 let ty = parse_type(&arg.ty)?;
-                if ident != "self" {
-                    let cfg = CfgExpr::Unconditional;
-                    let doc = Doc::new();
-                    let attrs = OtherAttrs::none();
-                    let visibility = Token![pub](ident.span());
-                    let name = pair(Namespace::default(), &ident, None, None);
-                    let colon_token = arg.colon_token;
-                    args.push_value(Var {
-                        cfg,
-                        doc,
-                        attrs,
-                        visibility,
-                        name,
-                        colon_token,
-                        ty,
-                    });
-                    if let Some(comma) = comma {
-                        args.push_punct(*comma);
-                    }
-                    continue;
+                let cfg = CfgExpr::Unconditional;
+                let doc = Doc::new();
+                let attrs = OtherAttrs::none();
+                let visibility = Token![pub](ident.span());
+                let name = pair(Namespace::default(), &ident, None, None);
+                let colon_token = arg.colon_token;
+                args.push_value(Var {
+                    cfg,
+                    doc,
+                    attrs,
+                    visibility,
+                    name,
+                    colon_token,
+                    ty,
+                });
+                if let Some(comma) = comma {
+                    args.push_punct(*comma);
                 }
-                if let Type::Ref(reference) = ty {
-                    if let Type::Ident(ident) = reference.inner {
-                        receiver = Some(Receiver {
-                            pinned: reference.pinned,
-                            ampersand: reference.ampersand,
-                            lifetime: reference.lifetime,
-                            mutable: reference.mutable,
-                            var: Token![self](ident.rust.span()),
-                            colon_token: arg.colon_token,
-                            ty: ident,
-                            shorthand: false,
-                            pin_tokens: reference.pin_tokens,
-                            mutability: reference.mutability,
-                        });
-                        continue;
-                    }
-                }
-                return Err(Error::new_spanned(arg, "unsupported method receiver"));
             }
         }
     }
@@ -756,6 +748,45 @@ fn parse_extern_verbatim_type(
     let type_token: Token![type] = input.parse()?;
     let ident: Ident = input.parse()?;
     let generics: Generics = input.parse()?;
+    let lifetimes = extern_type_lifetimes(cx, generics);
+    let lookahead = input.lookahead1();
+    if lookahead.peek(Token![=]) {
+        // type Alias = crate::path::to::Type;
+        parse_type_alias(
+            cx,
+            unparsed_attrs,
+            visibility,
+            type_token,
+            ident,
+            lifetimes,
+            input,
+            lang,
+            extern_block_cfg,
+            namespace,
+            attrs,
+        )
+    } else if lookahead.peek(Token![:]) {
+        // type Opaque: Bound2 + Bound2;
+        parse_extern_type_bounded(
+            cx,
+            unparsed_attrs,
+            visibility,
+            type_token,
+            ident,
+            lifetimes,
+            input,
+            lang,
+            trusted,
+            extern_block_cfg,
+            namespace,
+            attrs,
+        )
+    } else {
+        Err(lookahead.error())
+    }
+}
+
+fn extern_type_lifetimes(cx: &mut Errors, generics: Generics) -> Lifetimes {
     let mut lifetimes = Punctuated::new();
     let mut has_unsupported_generic_param = false;
     for pair in generics.params.into_pairs() {
@@ -788,45 +819,10 @@ fn parse_extern_verbatim_type(
             }
         }
     }
-    let lifetimes = Lifetimes {
+    Lifetimes {
         lt_token: generics.lt_token,
         lifetimes,
         gt_token: generics.gt_token,
-    };
-    let lookahead = input.lookahead1();
-    if lookahead.peek(Token![=]) {
-        // type Alias = crate::path::to::Type;
-        parse_type_alias(
-            cx,
-            unparsed_attrs,
-            visibility,
-            type_token,
-            ident,
-            lifetimes,
-            input,
-            lang,
-            extern_block_cfg,
-            namespace,
-            attrs,
-        )
-    } else if lookahead.peek(Token![:]) || lookahead.peek(Token![;]) {
-        // type Opaque: Bound2 + Bound2;
-        parse_extern_type_bounded(
-            cx,
-            unparsed_attrs,
-            visibility,
-            type_token,
-            ident,
-            lifetimes,
-            input,
-            lang,
-            trusted,
-            extern_block_cfg,
-            namespace,
-            attrs,
-        )
-    } else {
-        Err(lookahead.error())
     }
 }
 
@@ -928,9 +924,7 @@ fn parse_extern_type_bounded(
                 } else {
                     false
                 } => {}
-                bound @ TypeParamBound::Trait(_) | bound @ TypeParamBound::Lifetime(_) => {
-                    cx.error(bound, "unsupported trait");
-                }
+                bound => cx.error(bound, "unsupported trait"),
             }
 
             let lookahead = input.lookahead1();
@@ -1004,7 +998,7 @@ fn parse_impl(cx: &mut Errors, imp: ItemImpl) -> Result<Api> {
 
     if !imp.items.is_empty() {
         let mut span = Group::new(Delimiter::Brace, TokenStream::new());
-        span.set_span(imp.brace_token.span);
+        span.set_span(imp.brace_token.span.join());
         return Err(Error::new_spanned(span, "expected an empty impl block"));
     }
 
@@ -1151,7 +1145,7 @@ fn parse_type(ty: &RustType) -> Result<Type> {
         RustType::Path(ty) => parse_type_path(ty),
         RustType::Array(ty) => parse_type_array(ty),
         RustType::BareFn(ty) => parse_type_fn(ty),
-        RustType::Tuple(ty) if ty.elems.is_empty() => Ok(Type::Void(ty.paren_token.span)),
+        RustType::Tuple(ty) if ty.elems.is_empty() => Ok(Type::Void(ty.paren_token.span.join())),
         _ => Err(Error::new_spanned(ty, "unsupported type")),
     }
 }
@@ -1332,16 +1326,12 @@ fn parse_type_path(ty: &TypePath) -> Result<Type> {
 fn parse_type_array(ty: &TypeArray) -> Result<Type> {
     let inner = parse_type(&ty.elem)?;
 
-    let len_expr = if let Expr::Lit(lit) = &ty.len {
-        lit
-    } else {
+    let Expr::Lit(len_expr) = &ty.len else {
         let msg = "unsupported expression, array length must be an integer literal";
         return Err(Error::new_spanned(&ty.len, msg));
     };
 
-    let len_token = if let Lit::Int(int) = &len_expr.lit {
-        int.clone()
-    } else {
+    let Lit::Int(len_token) = &len_expr.lit else {
         let msg = "array length must be an integer literal";
         return Err(Error::new_spanned(len_expr, msg));
     };
@@ -1360,7 +1350,7 @@ fn parse_type_array(ty: &TypeArray) -> Result<Type> {
         inner,
         semi_token,
         len,
-        len_token,
+        len_token: len_token.clone(),
     })))
 }
 
@@ -1387,7 +1377,7 @@ fn parse_type_fn(ty: &TypeBareFn) -> Result<Type> {
             let (ident, colon_token) = match &arg.name {
                 Some((ident, colon_token)) => (ident.clone(), *colon_token),
                 None => {
-                    let fn_span = ty.paren_token.span;
+                    let fn_span = ty.paren_token.span.join();
                     let ident = format_ident!("arg{}", i, span = fn_span);
                     let colon_token = Token![:](fn_span);
                     (ident, colon_token)
@@ -1470,8 +1460,7 @@ fn parse_return_type(
 
 fn visibility_pub(vis: &Visibility, inherited: Span) -> Token![pub] {
     Token![pub](match vis {
-        Visibility::Public(vis) => vis.pub_token.span,
-        Visibility::Crate(vis) => vis.crate_token.span,
+        Visibility::Public(vis) => vis.span,
         Visibility::Restricted(vis) => vis.pub_token.span,
         Visibility::Inherited => inherited,
     })
